/** @file  pslib_strict.h
*
*  @brief Contains the complete list of header files for pslib while poisoning
*         the use of standard memory allocation routines.
*
*  This header file includes all the necessary header files for a user to
*  user all public functions within the pslib library.
*
*  @author Eric Van Alst, MHPCC
*
*  @version $Revision: 1.9 $ $Name: rel8_0 $
*  @date $Date: 2005/10/06 02:41:07 $
*
*  Copyright 2004-2005 Maui High Performance Computing Center, University of Hawaii
*/

#ifndef PS_LIB_STRICT_H
#define PS_LIB_STRICT_H

#ifdef PS_LIB_H  /* this is included from pslib.h, so don't poison anything */
#ifndef PS_ALLOW_MALLOC
#define PS_ALLOW_MALLOC
#endif // #ifndef PS_ALLOW_MALLOC
#else
#undef PS_ALLOW_MALLOC /* don't allow code to not poison malloc, i.e., strict poisioning */
#endif // #else

/// @defgroup astro Astrometry Functions
/// @{
#include "psTime.h"
#include "psCoord.h"
#include "psSphereOps.h"
/// @}

/// @defgroup db Database Functions
/// @{
#include "psDB.h"
/// @}

/// @defgroup fft Fourier Functions
/// @{
#include "psVectorFFT.h"
#include "psImageFFT.h"
/// @}

/// @defgroup fits FITS I/O Functions
/// @{
#include "psFits.h"
/// @}

/// @defgroup xml XML Functions
/// @{
#include "psXML.h"
/// @}

/// @defgroup imageops Image Operations
/// @{
#include "psImageConvolve.h"
#include "psImageGeomManip.h"
#include "psImagePixelExtract.h"
#include "psImagePixelManip.h"
#include "psImageStats.h"
#include "psImageStructManip.h"
#include "psImageMaskOps.h"
/// @}

/// @defgroup math Math Functions
/// @{
#include "psBinaryOp.h"
#include "psCompare.h"
#include "psConstants.h"
#include "psMatrix.h"
#include "psMinimize.h"
#include "psRandom.h"
#include "psPolynomial.h"
#include "psSpline.h"
#include "psStats.h"
#include "psUnaryOp.h"
/// @}

/// @defgroup mathtypes Mathematical Data Objects
/// @{
#include "psImage.h"
#include "psScalar.h"
#include "psVector.h"
/// @}

/// @defgroup sys System-Level Functions
/// @{
#include "psAbort.h"
#include "psConfigure.h"
#include "psError.h"
#include "psErrorCodes.h"
#include "psLogMsg.h"
#include "psMemory.h"
#include "psString.h"
#include "psTrace.h"
#include "psType.h"
/// @}

/// @defgroup types Basic Data Types
/// @{
#include "psArray.h"
#include "psBitSet.h"
#include "psHash.h"
#include "psList.h"
#include "psLookupTable.h"
#include "psMetadata.h"
#include "psMetadataConfig.h"
#include "psPixels.h"
/// @}

#endif // #ifndef PS_LIB_STRICT_H
